@@ -7,7 +7,7 @@
  *
  * Written by Russ Allbery <rra@stanford.edu>
  * Based on the original Kerberos support code by Roland Schemers
- * Copyright 2002, 2003, 2006, 2009
+ * Copyright 2002, 2003, 2006, 2009, 2010
  *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
@@ -22,10 +22,11 @@
  */
 static int
 cred_to_attr_encoding(WEBAUTH_KRB5_CTXTP *c, krb5_creds *creds,
-                      char **output, int *length, time_t *expiration)
+                      char **output, size_t *length, time_t *expiration)
 {
     WEBAUTH_ATTR_LIST *list;
-    int s, length_max;
+    int s;
+    size_t length_max;
 
     assert(c != NULL);
     assert(creds != NULL);
@@ -111,23 +112,23 @@ cred_to_attr_encoding(WEBAUTH_KRB5_CTXTP *c, krb5_creds *creds,
 
     /* Addresses. */
     if (creds->addresses) {
-        int num = 0, i;
+        unsigned int num = 0, i;
         char name[32];
         krb5_address **temp = creds->addresses;
 
         while (*temp++)
             num++;
-        s = webauth_attr_list_add_int32(list, CR_NUMADDRS, num, WA_F_NONE);
+        s = webauth_attr_list_add_uint32(list, CR_NUMADDRS, num, WA_F_NONE);
         if (s != WA_ERR_NONE)
             goto cleanup;
         for (i = 0; i < num; i++) {
-            sprintf(name, CR_ADDRTYPE, i);
+            snprintf(name, sizeof(name), CR_ADDRTYPE, i);
             s = webauth_attr_list_add_int32(list, name,
                                             creds->addresses[i]->addrtype,
                                             WA_F_COPY_NAME);
             if (s != WA_ERR_NONE)
                 goto cleanup;
-            sprintf(name, CR_ADDRCONT, i);
+            snprintf(name, sizeof(name), CR_ADDRCONT, i);
             s = webauth_attr_list_add(list, name,
                                       creds->addresses[i]->contents,
                                       creds->addresses[i]->length,
@@ -157,23 +158,23 @@ cred_to_attr_encoding(WEBAUTH_KRB5_CTXTP *c, krb5_creds *creds,
 
     /* Auth data. */
     if (creds->authdata) {
-        int num = 0, i;
+        unsigned int num = 0, i;
         char name[32];
         krb5_authdata **temp = creds->authdata;
 
         while (*temp++)
             num++;
-        s = webauth_attr_list_add_int32(list, CR_NUMAUTHDATA, num, WA_F_NONE);
+        s = webauth_attr_list_add_uint32(list, CR_NUMAUTHDATA, num, WA_F_NONE);
         if (s != WA_ERR_NONE)
             goto cleanup;
         for (i = 0; i < num; i++) {
-            sprintf(name, CR_AUTHDATATYPE, i);
+            snprintf(name, sizeof(name), CR_AUTHDATATYPE, i);
             s = webauth_attr_list_add_int32(list, name,
                                             creds->authdata[i]->ad_type,
                                             WA_F_COPY_NAME);
             if (s != WA_ERR_NONE)
                 goto cleanup;
-            sprintf(name, CR_AUTHDATACONT, i);
+            snprintf(name, sizeof(name), CR_AUTHDATACONT, i);
             s = webauth_attr_list_add(list, name,
                                       creds->authdata[i]->contents,
                                       creds->authdata[i]->length,
@@ -208,10 +209,12 @@ cred_to_attr_encoding(WEBAUTH_KRB5_CTXTP *c, krb5_creds *creds,
  */
 static int
 cred_from_attr_encoding(WEBAUTH_KRB5_CTXTP *c, char *input,
-                        int input_length, krb5_creds *creds)
+                        size_t input_length, krb5_creds *creds)
 {
     WEBAUTH_ATTR_LIST *list;
-    int s, f, length;
+    int s;
+    ssize_t f;
+    size_t length;
     void *data;
     char *buff;
 
@@ -305,10 +308,11 @@ cred_from_attr_encoding(WEBAUTH_KRB5_CTXTP *c, char *input,
      */
     webauth_attr_list_find(list, CR_NUMADDRS, &f);
     if (f != -1) {
-        int num = 0, i;
+        uint32_t num = 0;
+        unsigned int i;
         char name[32];
 
-        s = webauth_attr_list_get_int32(list, CR_NUMADDRS, &num, WA_F_NONE);
+        s = webauth_attr_list_get_uint32(list, CR_NUMADDRS, &num, WA_F_NONE);
         if (s != WA_ERR_NONE)
             goto cleanup;
 
@@ -328,13 +332,13 @@ cred_from_attr_encoding(WEBAUTH_KRB5_CTXTP *c, char *input,
                 goto cleanup;
 
             creds->addresses[i]->magic = KV5M_ADDRESS;
-            sprintf(name, CR_ADDRTYPE, i);
+            snprintf(name, sizeof(name), CR_ADDRTYPE, i);
             s = webauth_attr_list_get_int32(list, name,
                                             &creds->addresses[i]->addrtype,
                                             WA_F_NONE);
             if (s != WA_ERR_NONE)
                 goto cleanup;
-            sprintf(name, CR_ADDRCONT, i);
+            snprintf(name, sizeof(name), CR_ADDRCONT, i);
 
             s = webauth_attr_list_get(list, name, &data, &length,
                                       WA_F_COPY_VALUE);
@@ -372,10 +376,12 @@ cred_from_attr_encoding(WEBAUTH_KRB5_CTXTP *c, char *input,
     /* Auth data. */
     webauth_attr_list_find(list, CR_NUMAUTHDATA, &f);
     if (f != -1) {
-        int num = 0, i;
+        uint32_t num = 0;
+        unsigned int i;
         char name[32];
 
-        s = webauth_attr_list_get_int32(list, CR_NUMAUTHDATA, &num, WA_F_NONE);
+        s = webauth_attr_list_get_uint32(list, CR_NUMAUTHDATA, &num,
+                                         WA_F_NONE);
         if (s != WA_ERR_NONE)
             goto cleanup;
 
@@ -395,13 +401,13 @@ cred_from_attr_encoding(WEBAUTH_KRB5_CTXTP *c, char *input,
                 goto cleanup;
 
             creds->authdata[i]->magic = KV5M_AUTHDATA;
-            sprintf(name, CR_AUTHDATATYPE, i);
+            snprintf(name, sizeof(name), CR_AUTHDATATYPE, i);
             s = webauth_attr_list_get_int32(list, name,
                                             &creds->authdata[i]->ad_type,
                                             WA_F_NONE);
             if (s != WA_ERR_NONE)
                 goto cleanup;
-            sprintf(name, CR_AUTHDATACONT, i);
+            snprintf(name, sizeof(name), CR_AUTHDATACONT, i);
 
             s = webauth_attr_list_get(list, name, &data, &length,
                                       WA_F_COPY_VALUE);
@@ -437,9 +443,9 @@ cleanup:
 int
 webauth_krb5_mk_req_with_data(WEBAUTH_KRB5_CTXT *context,
                               const char *server_principal,
-                              char **output, int *length,
-                              char *in_data, int in_length,
-                              char **out_data, int *out_length)
+                              char **output, size_t *length,
+                              char *in_data, size_t in_length,
+                              char **out_data, size_t *out_length)
 {
     WEBAUTH_KRB5_CTXTP *c = (WEBAUTH_KRB5_CTXTP *) context;
     krb5_auth_context auth;
@@ -540,13 +546,13 @@ cleanup:
  */
 int
 webauth_krb5_rd_req_with_data(WEBAUTH_KRB5_CTXT *context,
-                              const char *req, int length,
+                              const char *req, size_t length,
                               const char *keytab_path,
                               const char *server_principal,
                               char **out_server_principal,
                               char **client_principal,
-                              int local, char *in_data, int in_length,
-                              char **out_data, int *out_length)
+                              int local, char *in_data, size_t in_length,
+                              char **out_data, size_t *out_length)
 {
     WEBAUTH_KRB5_CTXTP *c = (WEBAUTH_KRB5_CTXTP *) context;
     krb5_principal server;
@@ -579,7 +585,9 @@ webauth_krb5_rd_req_with_data(WEBAUTH_KRB5_CTXT *context,
             krb5_authenticator *ka;
 
             c->code = krb5_auth_con_getauthenticator(c->ctx, auth, &ka);
-            if (c->code == 0) {
+            if (c->code != 0)
+                *client_principal = NULL;
+            else {
                 int local_ok = 0;
 
                 if (local) {
@@ -590,8 +598,13 @@ webauth_krb5_rd_req_with_data(WEBAUTH_KRB5_CTXT *context,
                                                     sizeof(lname) - 1,
                                                     lname);
                     if (tcode == 0) {
-                        *client_principal = malloc(strlen(lname) + 1);
-                        strcpy(*client_principal, lname);
+                        *client_principal = strdup(lname);
+                        if (*client_principal == NULL) {
+                            krb5_free_authenticator(c->ctx, ka);
+                            krb5_auth_con_free(c->ctx, auth);
+                            s = WA_ERR_NO_MEM;
+                            goto done;
+                        }
                         local_ok = 1;
                     }
                 }
@@ -635,24 +648,19 @@ webauth_krb5_rd_req_with_data(WEBAUTH_KRB5_CTXT *context,
 
                 }
                 krb5_free_authenticator(c->ctx, ka);
-
-            } else
-                *client_principal = NULL;
+            }
             krb5_auth_con_free(c->ctx, auth);
         }
     }
 
+done:
     krb5_kt_close(c->ctx, keytab);
     krb5_free_principal(c->ctx, server);
-
     if (s == WA_ERR_NONE && c->code != 0)
         s = WA_ERR_KRB5;
-
-    if (s != WA_ERR_NONE) {
+    if (s != WA_ERR_NONE)
         if (out_server_principal && *out_server_principal != NULL)
             free(*out_server_principal);
-    }
-
     return s;
 }
 
@@ -665,7 +673,7 @@ webauth_krb5_rd_req_with_data(WEBAUTH_KRB5_CTXT *context,
  */
 int
 webauth_krb5_export_tgt(WEBAUTH_KRB5_CTXT *context,
-                        char **tgt, int *tgt_len, time_t *expiration)
+                        char **tgt, size_t *tgt_len, time_t *expiration)
 {
     WEBAUTH_KRB5_CTXTP *c = (WEBAUTH_KRB5_CTXTP *) context;
     krb5_principal tgtprinc, client;
