@@ -2,22 +2,27 @@
  * Random data generation functions.
  *
  * Written by Roland Schemers
- * Copyright 2002, 2006, 2009
+ * Copyright 2002, 2006, 2009, 2010
  *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
 
-#include <lib/webauthp.h>
+#include <config.h>
+#include <portable/system.h>
 
+#include <assert.h>
 #include <openssl/rand.h>
 
+#include <lib/webauth.h>
+
+
 /*
  * Generate num_bytes random bytes and store them in output.  This is
  * currently a simple wrapper around the OpenSSL function.
  */
 int
-webauth_random_bytes(char *output, int num_bytes)
+webauth_random_bytes(char *output, size_t num_bytes)
 {
     int s;
 
@@ -39,7 +44,7 @@ webauth_random_bytes(char *output, int num_bytes)
  * as webauth_random_bytes.
  */
 int
-webauth_random_key(char *key, int key_len)
+webauth_random_key(char *key, size_t key_len)
 {
     int s;
 
